package gus05.entity.gus.graphic.label.building.maillabel;


import java.awt.Color;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import gus05.framework.core.Entity;
import gus05.framework.core.Outside;
import gus05.framework.core.Service;
import gus05.framework.features.Graphic;

import javax.swing.JComponent;
import javax.swing.JLabel;



public class BuildingMailLabel implements Entity, Graphic {


    public String getName()				{return "gus.graphic.label.building.maillabel";}
    public String getCreationDate()		{return "2009.02.03";}



    
    private Service mailto_windows;
    private Service mailto_mac;
    private Service lingTooltip;
    
    private boolean isWindows;
    private boolean isMac;
    

    public BuildingMailLabel() throws Exception
    {
    	mailto_windows = Outside.service(this,"gus.env.windows.command.mailto");
    	mailto_mac = Outside.service(this,"gus.env.mac.command.mailto");
    	lingTooltip = Outside.service(this,"gus.ling.support.tooltip");
    	
    	String os = System.getProperty("os.name");
    	isWindows = os.toLowerCase().contains("windows");
    	isMac = os.toLowerCase().contains("mac");
    }



    public JComponent gui()
    {
    	if(isWindows) return new JMailLabel_windows();
    	if(isMac) return new JMailLabel_mac();
    	return new JLabel();
    }


    
    
    private void sendMailTo_windows(String mailAddress)
    {
    	try
    	{
    		if(isWindows) mailto_windows.give(mailAddress);
    	}
    	catch(Exception e)
    	{Outside.err(this,"sendMailTo_windows(String)",e);}
    }
    
    
    
    
    
    
    private void sendMailTo_mac(String mailAddress)
    {
    	try
    	{
    		if(isMac) mailto_mac.give(mailAddress);
    	}
    	catch(Exception e)
    	{Outside.err(this,"sendMailTo_mac(String)",e);}
    }
    
    
    
    
    
    
    private void lingTooltip(JLabel label)
    {
    	try
    	{
    		lingTooltip.give(label);
    	}
    	catch(Exception e)
    	{Outside.err(this,"lingTooltip(JLabel)",e);}
    }
    
    
    
    



    
    
    
    
    
    
    
    private class JMailLabel_windows extends JLabel implements MouseListener
    {
    	private Color COLOR_HIGHLIGHT = Color.GREEN;
    	private Color foreground;

    	public JMailLabel_windows()
    	{
    		super();
    		addMouseListener(this);
    		setFocusable(true);
    		foreground = getForeground();
    		setToolTipText("WEB_clickToMailTo");
    		lingTooltip(this);
    	}

    	public void setForeground(Color foreground)
    	{
    		this.foreground = foreground;
    		super.setForeground(foreground);
    	}

    	public void mouseClicked(MouseEvent e) {}
    	public void mouseReleased(MouseEvent e) {}
    	public void mouseEntered(MouseEvent e)
    	{
    		super.setForeground(COLOR_HIGHLIGHT);
    	}
    	public void mouseExited(MouseEvent e)
    	{
    		super.setForeground(foreground);
    	}
    	public void mousePressed(MouseEvent e)
    	{
    		super.setForeground(foreground);
    		sendMailTo_windows(getText());
    	}
    }
    
    
    
    
    
    
    
    
    
    
    private class JMailLabel_mac extends JLabel implements MouseListener
    {
    	private Color COLOR_HIGHLIGHT = Color.RED;
    	private Color foreground;
    	
    	public JMailLabel_mac()
    	{
    		super();
    		addMouseListener(this);
    		setFocusable(true);
    		foreground = getForeground();
    		setToolTipText("WEB_clickToMailTo");
    		lingTooltip(this);
    	}

    	public void setForeground(Color foreground)
    	{
    		this.foreground = foreground;
    		super.setForeground(foreground);
    	}

    	public void mouseClicked(MouseEvent e) {}
    	public void mouseReleased(MouseEvent e) {}
    	public void mouseEntered(MouseEvent e)
    	{
    		super.setForeground(COLOR_HIGHLIGHT);
    	}
    	public void mouseExited(MouseEvent e)
    	{
    		super.setForeground(foreground);
    	}
    	public void mousePressed(MouseEvent e)
    	{
    		super.setForeground(foreground);
    		sendMailTo_mac(getText());
    	}
    }
}

