package gus05.entity.gus.graphic.textcomp.action.f.highlight1;


import java.awt.Color;
import java.awt.Event;
import java.awt.event.KeyEvent;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.Action;
import javax.swing.JOptionPane;
import javax.swing.KeyStroke;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.text.DefaultHighlighter;
import javax.swing.text.Highlighter;
import javax.swing.text.JTextComponent;

import gus05.framework.core.Entity;
import gus05.framework.core.Outside;
import gus05.framework.features.Execute;
import gus05.framework.features.Give;
import gus05.framework.features.Transform;
import gus05.framework.resources.ActionBuilder;



public class Highlight1 implements Entity, Give, Transform {


    public String getName()				{return "gus.graphic.textcomp.action.f.highlight1";}
    public String getCreationDate()		{return "2006.10.06";}



    private ActionBuilder actionBuilder;
    private KeyStroke highlightKey;
    
    public Highlight1() throws Exception
    {
        actionBuilder = (ActionBuilder) Outside.resource(this,"app.actionbuilder");
        highlightKey = KeyStroke.getKeyStroke(KeyEvent.VK_F,Event.CTRL_MASK);
    }



    public void give(Object obj) throws Exception
    {
        JTextComponent comp = (JTextComponent)obj;
        HighlightExecute high = new HighlightExecute(comp);
        Action action = actionBuilder.build("TEXTCOMP_highlight",high);
        comp.getInputMap().put(highlightKey, action);
    }



    public Object transform(Object obj) throws Exception
    {
        HighlightExecute high = new HighlightExecute((JTextComponent)obj);
        return actionBuilder.build("TEXTCOMP_highlight",high);
    }


    
    
    
    
    
    
    private class HighlightPainter extends DefaultHighlighter.DefaultHighlightPainter
    {
        public HighlightPainter()
        {super(Color.GREEN);}
    }
    
    
    
    
    private class HighlightExecute implements Execute, DocumentListener
    {
        private JTextComponent comp;
        private Highlighter high;
        private HighlightPainter painter;
        private boolean activated = false;
        private String exp;
        private boolean found;
        
        private String selected()
        {return comp.getSelectedText();}
        
        private boolean hasSelection()
        {return selected()!=null && !selected().equals("");}
        
        
        
        public HighlightExecute(JTextComponent comp)
        {
            this.comp = comp;
            high = comp.getHighlighter();
            painter = new HighlightPainter();
            comp.getDocument().addDocumentListener(this);
        }
        
        
        public void activate()
        {
            exp = retrieveExpression();
            if(exp==null || exp.equals(""))return;
            
            addHighlights();
            if(!found)return;
            
            int p = comp.getCaretPosition();
            comp.select(p,p);
            activated = true;
        }
        
        
        public void disactivate()
        {
            removeHighlights();
            activated = false;
            
            if(hasSelection()) activate();
        }
        
        
        public void update()
        {
            removeHighlights();
            if(activated)addHighlights();   
        }

        
        public void execute() throws Exception
        {
            if(activated) disactivate();
            else activate();
        }
      
        
        
        private String retrieveExpression()
        {
            if(hasSelection()) return selected();
            return JOptionPane.showInputDialog(null,"Please, type an expression:",
                    "Search expression",JOptionPane.PLAIN_MESSAGE);
        }
        
        
        
        public void addHighlights()
        {
            try
            {
                Pattern p = Pattern.compile(Pattern.quote(exp),Pattern.DOTALL);
                String text = comp.getText();
                Matcher m = p.matcher(text);
                
                found = false;
                while(m.find())
                {
                    found = true;
                    int start = m.start();
                    int end = m.end();
                    high.addHighlight(start,end,painter);
                }
            }
            catch(Exception e)
            {Outside.err(Highlight1.this,"addHighlights()",e);}
        }
        
        
        
        
        public void removeHighlights()
        {
            Highlighter.Highlight[] h = high.getHighlights();
            for(int i=0;i<h.length;i++)
            if(h[i].getPainter().equals(painter))
            {
                HighlightPainter p = (HighlightPainter)h[i].getPainter();
                high.removeHighlight(h[i]);
            }
        }

        
        
        public void insertUpdate(DocumentEvent e) {update();}
        public void removeUpdate(DocumentEvent e) {update();}
        public void changedUpdate(DocumentEvent e) {update();}
    }

}

