package gus05.entity.gus.graphic.textcomp.action.r.applyrule1;


import java.awt.Event;
import java.awt.event.KeyEvent;

import javax.swing.Action;
import javax.swing.JOptionPane;
import javax.swing.KeyStroke;
import javax.swing.text.JTextComponent;

import gus05.framework.core.Entity;
import gus05.framework.core.Outside;
import gus05.framework.core.Service;
import gus05.framework.features.Execute;
import gus05.framework.features.Give;
import gus05.framework.features.Transform;
import gus05.framework.resources.ActionBuilder;



public class ApplyRule1 implements Entity, Give, Transform {


    public String getName()				{return "gus.graphic.textcomp.action.r.applyrule1";}
    public String getCreationDate()		{return "2006.09.25";}

    

    private Service generator;
    private ActionBuilder actionBuilder;
    private KeyStroke ruleKey;
    
    
    public ApplyRule1() throws Exception
    {
        generator = Outside.service(this,"gus.app.servicegenerator2");
        actionBuilder = (ActionBuilder) Outside.resource(this,"app.actionbuilder");
        ruleKey = KeyStroke.getKeyStroke(KeyEvent.VK_R,Event.CTRL_MASK);
    }



    public void give(Object obj) throws Exception
    {
        JTextComponent comp = (JTextComponent)obj;
        RuleHandler handler = new RuleHandler(comp);
        Action action = actionBuilder.build("TEXTCOMP_applyrule",handler);
        comp.getInputMap().put(ruleKey, action);
    }



    public Object transform(Object obj) throws Exception
    {
        RuleHandler handler = new RuleHandler((JTextComponent)obj);
        return actionBuilder.build("TEXTCOMP_applyrule",handler);
    }


    
    
    
    
    
    private String getSelectedText(JTextComponent comp)
    {
        if(comp.getSelectedText()==null)return "";
        return comp.getSelectedText();
    }
    
    
    private void modifyText(JTextComponent comp, Transform t) throws Exception
    {
        String newText = (String)t.transform(comp.getText());
        comp.setText(newText);
    }
    
    
    private void modifySelectedText(JTextComponent comp, Transform t) throws Exception
    {
        String newText = (String)t.transform(comp.getSelectedText());
        int start = comp.getSelectionStart();
        comp.replaceSelection(newText);
        comp.select(start,start+newText.length());
    }


    
    
    
    private class RuleHandler implements Execute
    {
        private JTextComponent comp;
        public RuleHandler(JTextComponent comp)
        {this.comp = comp;}
        
        public void execute() throws Exception
        {
            String rule = JOptionPane.showInputDialog(null,"Rule:","Apply rule",JOptionPane.PLAIN_MESSAGE);
            if(rule==null) return;
            
            if(rule.contains("<>")) applyTransformRule2(rule);
            else applyTransformRule1(rule);
        }        
        
        private void applyTransformRule1(String rule) throws Exception
        {
            Transform t = (Transform)generator.transform(rule);
            if(getSelectedText(comp).equals("")) modifyText(comp,t);
            else modifySelectedText(comp,t);
        }        
        
        private void applyTransformRule2(String rule) throws Exception
        {
            rule = rule.replace("<>",getSelectedText(comp));
            Transform t = (Transform)generator.transform(rule);
            modifyText(comp,t);
        }
    }
}

