package gus05.entity.gus.graphic.textcomp.action.z.undo;


import java.awt.Event;
import java.awt.event.KeyEvent;
import java.util.Vector;

import javax.swing.Action;
import javax.swing.KeyStroke;
import javax.swing.event.UndoableEditEvent;
import javax.swing.text.JTextComponent;
import javax.swing.undo.UndoManager;

import gus05.framework.core.Entity;
import gus05.framework.core.Outside;
import gus05.framework.features.Execute;
import gus05.framework.features.Give;
import gus05.framework.features.Transform;
import gus05.framework.resources.ActionBuilder;



public class Undo implements Entity, Give, Transform {


    public String getName()				{return "gus.graphic.textcomp.action.z.undo";}
    public String getCreationDate()		{return "2006.08.11";}


    
    private ActionBuilder actionBuilder;
    private KeyStroke undoKey;
    
    
    public Undo() throws Exception
    {
        actionBuilder = (ActionBuilder) Outside.resource(this,"app.actionbuilder");
        undoKey = KeyStroke.getKeyStroke(KeyEvent.VK_Z,Event.CTRL_MASK);
    }

    

    public Object transform(Object obj) throws Exception
	{
		JTextComponent comp = (JTextComponent)obj;
		UndoManager0 handler = new UndoManager0(comp);
		
		Vector vec = new Vector();
		vec.add(handler.getActionUndo());
		vec.add(handler.getActionRedo());
		vec.add(handler.getActionPurge());
		
		return vec;
	}



    public void give(Object obj) throws Exception
    {
        JTextComponent comp = (JTextComponent)obj;
        new UndoManager0(comp);
    }


    
    
    
    
    
    
    
    
    
    private class UndoManager0 extends UndoManager
    {
        private JTextComponent component;
        
        private Action actionUndo;
        private Action actionRedo;
        private Action actionPurge;
        
        public Action getActionUndo()
        {return actionUndo;}
        
        public Action getActionRedo()
        {return actionRedo;}
        
        public Action getActionPurge()
        {return actionPurge;}
        
        
        
        
        public UndoManager0(JTextComponent component)
        {
        	super();
            this.component = component;
            component.getDocument().addUndoableEditListener(this);
            
            initActions();
            update();
        }
        
        
        
        private void initActions()
        {
            actionUndo = actionBuilder.build("TEXT_undo",new Execute(){
                public void execute() throws Exception {undo();}});
            
            actionRedo = actionBuilder.build("TEXT_redo",new Execute(){
                public void execute() throws Exception {redo();}});
            
            actionRedo = actionBuilder.build("TEXT_purgeUndo",new Execute(){
                public void execute() throws Exception {discardAllEdits();}});
            
            component.getInputMap().put(undoKey, actionUndo);
        }
         
        
        public void undoableEditHappened(UndoableEditEvent evt)
        {
            super.undoableEditHappened(evt);
            update();
        }
        
        public void undo()
        {
        	if(!canUndo()) return;
        	super.undo();
        	update();
        }
        
        public void redo()
        {
        	if(!canRedo()) return;
        	super.redo();
        	update();
        }

        private void update()
        {
           actionUndo.setEnabled(canUndo());
           actionRedo.setEnabled(canRedo());
        }
    }
}

