package gus05.entity.gus.java.jarinspector.app1.engine;


import java.io.File;
import java.io.InputStream;
import java.io.PrintStream;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.Map;
import java.util.Properties;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;

import javax.swing.JLabel;

import gus05.framework.core.Entity;
import gus05.framework.core.Outside;
import gus05.framework.core.Service;
import gus05.framework.features.Filter;
import gus05.framework.features.Provide;
import gus05.framework.features.Register;
import gus05.framework.features.Retrieve;
import gus05.framework.tools.DefaultSupport;



public class Engine extends DefaultSupport implements Entity, Register, Retrieve, Provide, Runnable {


    public String getName()				{return "gus.java.jarinspector.app1.engine";}
    public String getCreationDate()		{return "2009.02.27";}


    private Service filteringMap;
    private Service filterRule;
    private PrintStream out;
    
    private JLabel label_currentScan;
    
    private File scanDir;
    
    private String fieldInfo;
    private String valueInfo;
    private Filter entryInfoFilter;
    private boolean entryInfoIgnored = false;
    
    private File currentJar;
    private JarEntry currentEntry;
    private String currentLine;
    
    private boolean stop = false;
    
    


    public Engine() throws Exception
    {
    	filteringMap = Outside.service(this,"gus.collection.map.transform.filteringmap1");
    	filterRule = Outside.service(this,"gus.filter.string.build.rule1");
    	out = (PrintStream) Outside.resource(this,"out");
    	label_currentScan = new JLabel();
    }
    
    
    private Filter buildFilter(String rule) throws Exception
    {return (Filter) filterRule.transform(rule);}



    public void register(String key, Object obj) throws Exception
    {
    	if(key.equals("scanDir")) {scanDir = (File) obj;return;}
    	if(key.equals("entryInfo")) {initEntryInfo((String) obj);return;}
    	if(key.equals("fieldInfo")) {fieldInfo = (String) obj;return;}
    	if(key.equals("valueInfo")) {valueInfo = (String) obj;return;}
    	if(key.equals("label_currentScan")) {label_currentScan = (JLabel) obj;return;}
    	if(key.equals("stop")) {stop = true;return;}
    	
        throw new Exception("Unknown key: "+key);
    }
    
    
    
    
    private void initEntryInfo(String entryInfo) throws Exception
    {
    	entryInfoIgnored = entryInfo.startsWith("ignore:");
		entryInfoFilter = entryInfoIgnored?null:buildFilter(entryInfo);
    }
    
    



    public Object retrieve(String key) throws Exception
    {
    	if(key.equals("label_currentScan")) return label_currentScan;
    	if(key.equals("currentJar")) return currentJar;
    	if(key.equals("currentEntry")) return currentEntry;
    	if(key.equals("currentLine")) return currentLine;
    	
    	if(key.equals("keys"))
    		return new String[]{"label_currentScan","currentJar","currentEntry","currentLine"};
        throw new Exception("Unknown key: "+key);
    }
    
    
    public Object provide() throws Exception
    {return currentLine;}



    public void run()
    {
    	try
    	{
    		if(scanDir==null) throw new Exception("Scan dir has not been initialized yet");
    		if(fieldInfo==null) throw new Exception("Field info has not been initialized yet");
    		if(valueInfo==null) throw new Exception("Value info has not been initialized yet");
    		
    		stop = false;
    		
    		filteringMap.register("info",new String[]{fieldInfo,valueInfo});
    		out.println("start scan with field="+fieldInfo+" & value="+valueInfo);
    		analyzeDir(scanDir);
    	}
    	catch(Exception e)
    	{Outside.err(this,"run()",e);}
    }


    
    
    private void analyzeDir(File dir)
    {
    	label_currentScan.setText(dir.getAbsolutePath());
    	
    	File[] f = dir.listFiles();
    	for(int i=0;i<f.length;i++)
    	{
    		if(f[i].isDirectory()) analyzeDir(f[i]);
    		if(f[i].isFile()) analyzeFile(f[i]);
    		if(stop) return;
    	}
    }
    
    
    
    private void analyzeFile(File file)
    {
    	label_currentScan.setText(file.getAbsolutePath());
    	if(file.getName().toLowerCase().endsWith(".jar"))
    		analyzeJar(file);
    }


    
    
    private void analyzeJar(File file)
    {
    	try
    	{
    		currentJar = file;
    		JarFile jarFile = new JarFile(file,true,JarFile.OPEN_READ);
    		Enumeration en = jarFile.entries();
            while(en.hasMoreElements())
            {
                JarEntry entry = (JarEntry)en.nextElement();
                String entryName = entry.getName();
                
                if(entryName.toLowerCase().endsWith(".properties"))
                if(entryInfoIgnored || entryInfoFilter.filter(entryName))
                {
                	currentEntry = entry;
                	InputStream is = jarFile.getInputStream(entry);
                	Properties prop = new Properties();
                	prop.load(is);
                	is.close();
                	
                	analyzeProp(prop);
                }
                if(stop) return;
            }
    	}
    	catch(Exception e)
    	{Outside.err(this,"analyzeJar(File)",e);}
    }
    
    

    private void analyzeProp(Properties prop)
    {
    	try
    	{
    		Map m = (Map) filteringMap.transform(prop);
    		if(m.isEmpty()) return;
    		
    		String path = currentJar+"\t"+currentEntry.getName();
    		Iterator it = m.keySet().iterator();
    		while(it.hasNext())
    		{
    			String key = (String) it.next();
    			String value = (String) m.get(key);
    			String line = path+"\t"+key+"\t"+value;
    			
    			currentLine = line;
    			resultFound();
    		}
    	}
    	catch(Exception e)
    	{Outside.err(this,"analyzeProp(Properties)",e);}
    }
	
    
    
    private void resultFound()
    {send(this,"resultFound()");}

}

