package gus05.manager.gus.endorf.layer3.system1;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Map;

import javax.swing.JOptionPane;

import gus05.manager.gus.endorf.layer2.EndorfSystem;
import gus05.manager.gus.endorf.layer2.datasupport.SupportMap;
import gus05.manager.gus.endorf.layer3.loading.*;
import gus05.manager.gus.endorf.layer3.tools.Tools_javaVersion;


public class System1 extends EndorfSystem {

    
    public static final String INTERNAL_MAP_FILES               = "app.filemap";
    public static final String INTERNAL_MAP_URLS                = "app.urlmap";
    public static final String INTERNAL_MAP_SOUNDS              = "app.soundmap";
    public static final String INTERNAL_MAP_ICONS               = "app.gui.iconmap";
    public static final String INTERNAL_MAP_LINGSTRING          = "app.lingstringmap";
    public static final String INTERNAL_MAP_INFOENTITY          = "app.entityinfomap";
    public static final String INTERNAL_MAP_PUBLICKEYS          = "app.security.publickeymap";
    
    public static final String INTERNAL_FILE_ROOTDIR            = "app.rootdir";
    public static final String INTERNAL_OBJ_ENTITYSECUREDLIST   = "app.entitysecuredlist";
    
    
    // path for directories
    public static final String PATH_ROOTDIR                 = "path.rootdir";
    public static final String PATH_ENTITYDIR               = "path.entitydir";
    public static final String PATH_LOGDIR                  = "path.logdir";
    public static final String PATH_ICONDIR                 = "path.icondir";
    public static final String PATH_SOUNDDIR                = "path.sounddir";
    public static final String PATH_LINGDIR                 = "path.lingdir";
    public static final String PATH_PUBLICKEYDIR            = "path.publickeydir";
    public static final String PATH_WORKSPACE               = "path.workspace";
    public static final String PATH_POOLDIR                 = "path.pooldir";
    
    // path for files
    public static final String PATH_PROPERTYFILE            = "path.propertyfile";
    public static final String PATH_MAPPINGFILE             = "path.mappingfile";
    public static final String PATH_LOADFILE                = "path.loadfile";
    public static final String PATH_STARTFILE               = "path.startfile";
    public static final String PATH_LOGFILE                 = "path.logfile";
    public static final String PATH_ICONFILESYSTEM          = "path.iconfilesystem";
    
    // entity info
    public static final String ENTITYINFO_AUTHOR                        = "author";
    public static final String ENTITYINFO_SIGN                          = "sign";
    public static final String ENTITYINFO_CLASSPATH                     = "classpath";
    public static final String ENTITYINFO_JAVAVERSION                   = "javaversion";
    public static final String ENTITYINFO_MULTIPLICITY                  = "multiplicity";
    public static final String ENTITYINFO_CREATIONDATE                  = "creationdate";
    public static final String ENTITYINFO_DEPLOYINGDATE                 = "deployingdate";
    public static final String ENTITYINFO_DEPLOYER                      = "deployer";
    
    public static final String ENTITYINFO_MULTIPLICITYVALUE_MANY        = "many";
    public static final String ENTITYINFO_MULTIPLICITYVALUE_UNIQUE      = "unique";
    public static final String ENTITYINFO_AUTHOR_UNKNOWN                = "unknown";
    
    
    // property keys
    public static final String PROPKEY_APPUSER              = "app.user";
    public static final String PROPKEY_APPSOUNDENABLED      = "app.sound.enabled";
    public static final String PROPKEY_APPJAVAVERSION       = "app.javaversion";
    public static final String PROPKEY_APPCREATIONDATE      = "app.creationdate";
    public static final String PROPKEY_APPVERSION           = "app.version";
    public static final String PROPKEY_APPTITLE             = "app.title";
    
    public static final String PROPKEY_DEBUGPROP         = "debug.display.prop";
    public static final String PROPKEY_DEBUGFILES        = "debug.display.files";
    public static final String PROPKEY_DEBUGURLS         = "debug.display.urls";
    public static final String PROPKEY_DEBUGMAPPING      = "debug.display.mapping";
    public static final String PROPKEY_DEBUGSTART        = "debug.display.start";
    public static final String PROPKEY_DEBUGINTERNALS    = "debug.display.internals";
    public static final String PROPKEY_DEBUGPROVIDERS    = "debug.display.providers";
    
    public static final String FILENAME_DEBUG = "debug.properties";
    
    
    
    
    
    
    
    
    public static File rootDir;
    public static File propFile;
    
    public static Map iconMap;
    public static Map soundMap;
    public static Map lingMap;
    public static Map fileMap;
    public static Map urlMap;
    public static Map infoMap;
    public static Map pKeyMap;
    
    public static boolean initOutside = false;
    
    public static EntityManager entityManager;
    public static FileManager fileManager;
    public static UrlManager urlManager;
    
    
    
    
    public static void init_1()
    {
        changePhase(1);
        m1("init_1()","LOAD RESOURCES");
        
        iconMap = new SupportMap("iconMap");
        soundMap = new SupportMap("soundMap");
        lingMap = new SupportMap("lingMap");
        fileMap = new SupportMap("fileMap");
        urlMap = new SupportMap("urlMap");
        infoMap = new SupportMap("infoMap");
        pKeyMap = new SupportMap("pKeyMap");
        
        putInternal(INTERNAL_MAP_ICONS,iconMap);
        putInternal(INTERNAL_MAP_SOUNDS,soundMap);
        putInternal(INTERNAL_MAP_LINGSTRING,lingMap);
        putInternal(INTERNAL_MAP_FILES,fileMap);
        putInternal(INTERNAL_MAP_URLS,urlMap);
        putInternal(INTERNAL_MAP_INFOENTITY,infoMap);
        putInternal(INTERNAL_MAP_PUBLICKEYS,pKeyMap);
        
        entityManager = new EntityManager();
        fileManager = new FileManager();
        urlManager = new UrlManager();
        
        
        
        initOutside = hasArg("init");
        if(initOutside) w1("init_1()","initOutside activated");
        
        
        new Loader_root();
        m1("init_1()","root dir = "+System1.rootDir);
        m1("init_1()","properties file = "+System1.propFile);
        
        
        if(debug)new Loader_debug();
        
        Loader_files l_files = new Loader_files();
        new Loader_prop();
        l_files.loadFromProperties();
        
        new Loader_urls();
        new Loader_mapping();
        new Loader_start();
        new Loader_icons();
        new Loader_sounds();
        new Loader_lings();
        new Loader_pKeys();
        
        checkAppInfo();
        
        m1("init_1()",properties.size()+" properties have been loaded");
        m1("init_1()",fileMap.size()+" files have been loaded");
        m1("init_1()",urlMap.size()+" urls have been loaded");
        m1("init_1()",mapping.size()+" mappings have been loaded");
        m1("init_1()",startList.size()+" starts have been loaded");
        
        if(iconMap.size()>0) m1("init_1()",iconMap.size()+" icons have been loaded");
        if(soundMap.size()>0) m1("init_1()",soundMap.size()+" sounds have been loaded");
        if(lingMap.size()>0) m1("init_1()",lingMap.size()+" lings have been loaded");
        if(pKeyMap.size()>0) m1("init_1()",pKeyMap.size()+" public keys have been loaded");
        if(mapping.isEmpty()) w1("init_1()","NO MAPPING FOUND");
    } 
    
    
    
    
    
    
    public static void validateRoot()
    {
        if(rootDir!=null) putFile(PATH_ROOTDIR,rootDir);
        if(propFile!=null) putFile(PATH_PROPERTYFILE,propFile);
    }
    
    
    
    
    
    
    
    
    private static void checkAppInfo()
    {
        if(hasProperty(PROPKEY_APPTITLE))
        m1("checkAppInfo()","App title: "+getProperty(PROPKEY_APPTITLE));

        if(hasProperty(PROPKEY_APPVERSION))
        m1("checkAppInfo()","App version: "+getProperty(PROPKEY_APPVERSION));

        if(hasProperty(PROPKEY_APPCREATIONDATE))
        m1("checkAppInfo()","App creation date: "+getProperty(PROPKEY_APPCREATIONDATE));

        if(hasProperty(PROPKEY_APPJAVAVERSION))
        {
            String javaVersion = getProperty(PROPKEY_APPJAVAVERSION);
            m1("checkAppInfo()","App java version: "+javaVersion);
            
            if(!Tools_javaVersion.isJavaVersionSupported(javaVersion))
            {
            	String message = "app java required version ("+javaVersion+") is not supported\n" +
                "Current version is "+System.getProperty("java.version");
            	String question = "Do you wish to continue anyway ?";
            	
            	int value = JOptionPane.showConfirmDialog(null,message+"\n"+question,
            			"COMPATIBILITY WARNING",
            			JOptionPane.YES_NO_OPTION,
            			JOptionPane.WARNING_MESSAGE);
            	
            	if(value==JOptionPane.NO_OPTION)
            	FATAL_ERROR(System1.class,"checkAppInfo()",message);
            }
        }
    }
    
    
    
    
    
    
    
    
    
    
    
    
    //********************************************
    // files
    
    public static boolean hasFile(String key)
    {return fileMap.containsKey(key);}
    
    public static File getFile(String key)
    {return fileMap.containsKey(key)?(File) fileMap.get(key):null;}
    
    public static void putFile(String key, File f)
    {
        if(!key.startsWith("path."))
        w1("putFile(String,File)","fileID is expected to start with \"path.\": "+key);
        fileMap.put(key,f);
    }
    
    public static File searchFile(String key)
    {return fileManager.searchFile(key);}
    
    
    
    
    
    
    //********************************************
    // urls
    
    public static boolean hasURL(String key)
    {return urlMap.containsKey(key);}
    
    public static URL url(String key)
    {return (URL) urlMap.get(key);}
    
    public static void putURL(String key, URL url)
    {
        if(!key.startsWith("url."))
        w1("putURL(String,URL)","urlID is expected to start with \"url.\": "+key);
        urlMap.put(key,url);
    }
    
    public static URL findURL(String key) throws MalformedURLException
    {
        if(urlMap.containsKey(key)) return url(key);
        urlManager.checkURL(key);
        if(urlMap.containsKey(key)) return url(key);
        
        if(key.startsWith("url.")) w1("findURL(String)","urlID not found: "+key);
        return new URL(key);
    }
    
    
    
    
    
    
    
    
    //********************************************
    // printing
    
    private static void m1(String id, String message)
    {printMessage(System1.class,id,message);}
    
    private static void w1(String id, String message)
    {printWarning(System1.class,id,message);}
}
